@@ -1,11 +1,15 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.engine;
 
+import java.util.Arrays;
+import java.util.Comparator;
 import java.util.HashMap;
+import java.util.Map;
+import java.util.Map.Entry;
 
 import org.h2.api.ErrorCode;
 import org.h2.message.DbException;
@@ -39,6 +43,16 @@ public class SettingsBase {
         }
     }
 
+    /**
+     * Set an entry in the key-value pair.
+     *
+     * @param key the key
+     * @param value the value
+     */
+    void set(String key, boolean value) {
+        settings.put(key, Boolean.toString(value));
+    }
+
     /**
      * Get the setting for the given key.
      *
@@ -47,7 +61,7 @@ public class SettingsBase {
      * @return the setting
      */
     protected int get(String key, int defaultValue) {
-        String s = get(key, "" + defaultValue);
+        String s = get(key, Integer.toString(defaultValue));
         try {
             return Integer.decode(s);
         } catch (NumberFormatException e) {
@@ -104,4 +118,16 @@ public class SettingsBase {
         return settings;
     }
 
+    /**
+     * Get all settings in alphabetical order.
+     *
+     * @return the settings
+     */
+    public Entry<String, String>[] getSortedSettings() {
+        @SuppressWarnings("unchecked")
+        Map.Entry<String, String>[] entries = settings.entrySet().toArray(new Map.Entry[0]);
+        Arrays.sort(entries, Comparator.comparing(Entry::getKey));
+        return entries;
+    }
+
 }
