@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2018 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.samples;
@@ -28,14 +28,14 @@ public class FunctionMultiReturn {
      * command line.
      *
      * @param args the command line parameters
+     * @throws Exception on failure
      */
     public static void main(String... args) throws Exception {
         Class.forName("org.h2.Driver");
         Connection conn = DriverManager.getConnection(
                 "jdbc:h2:mem:", "sa", "");
         Statement stat = conn.createStatement();
-        stat.execute("CREATE ALIAS P2C " +
-                "FOR \"org.h2.samples.FunctionMultiReturn.polar2Cartesian\" ");
+        stat.execute("CREATE ALIAS P2C FOR 'org.h2.samples.FunctionMultiReturn.polar2Cartesian'");
         PreparedStatement prep = conn.prepareStatement(
                 "SELECT X, Y FROM P2C(?, ?)");
         prep.setDouble(1, 5.0);
@@ -49,8 +49,7 @@ public class FunctionMultiReturn {
 
         stat.execute("CREATE TABLE TEST(ID IDENTITY, R DOUBLE, A DOUBLE)");
         stat.execute("INSERT INTO TEST(R, A) VALUES(5.0, 0.5), (10.0, 0.6)");
-        stat.execute("CREATE ALIAS P2C_SET " +
-                "FOR \"org.h2.samples.FunctionMultiReturn.polar2CartesianSet\" ");
+        stat.execute("CREATE ALIAS P2C_SET FOR 'org.h2.samples.FunctionMultiReturn.polar2CartesianSet'");
         rs = conn.createStatement().executeQuery(
                 "SELECT * FROM P2C_SET('SELECT * FROM TEST')");
         while (rs.next()) {
@@ -62,8 +61,7 @@ public class FunctionMultiReturn {
                     " (x=" + x + ", y="+y+")");
         }
 
-        stat.execute("CREATE ALIAS P2C_A " +
-                "FOR \"org.h2.samples.FunctionMultiReturn.polar2CartesianArray\" ");
+        stat.execute("CREATE ALIAS P2C_A FOR 'org.h2.samples.FunctionMultiReturn.polar2CartesianArray'");
         rs = conn.createStatement().executeQuery(
                 "SELECT R, A, P2C_A(R, A) FROM TEST");
         while (rs.next()) {
@@ -71,8 +69,8 @@ public class FunctionMultiReturn {
             double a = rs.getDouble(2);
             Object o = rs.getObject(3);
             Object[] xy = (Object[]) o;
-            double x = ((Double) xy[0]).doubleValue();
-            double y = ((Double) xy[1]).doubleValue();
+            double x = (Double) xy[0];
+            double y = (Double) xy[1];
             System.out.println("(r=" + r + " a=" + a + ") :" +
                     " (x=" + x + ", y=" + y + ")");
         }
@@ -108,8 +106,8 @@ public class FunctionMultiReturn {
         rs.addColumn("X", Types.DOUBLE, 0, 0);
         rs.addColumn("Y", Types.DOUBLE, 0, 0);
         if (r != null && alpha != null) {
-            double x = r.doubleValue() * Math.cos(alpha.doubleValue());
-            double y = r.doubleValue() * Math.sin(alpha.doubleValue());
+            double x = r * Math.cos(alpha);
+            double y = r * Math.sin(alpha);
             rs.addRow(x, y);
         }
         return rs;
@@ -124,10 +122,10 @@ public class FunctionMultiReturn {
      * @param alpha the angle
      * @return an array two values: x and y
      */
-    public static Object[] polar2CartesianArray(Double r, Double alpha) {
-        double x = r.doubleValue() * Math.cos(alpha.doubleValue());
-        double y = r.doubleValue() * Math.sin(alpha.doubleValue());
-        return new Object[]{x, y};
+    public static Double[] polar2CartesianArray(Double r, Double alpha) {
+        double x = r * Math.cos(alpha);
+        double y = r * Math.sin(alpha);
+        return new Double[]{x, y};
     }
 
     /**
@@ -138,6 +136,7 @@ public class FunctionMultiReturn {
      * @param conn the connection
      * @param query the query
      * @return a result set with the coordinates
+     * @throws SQLException on failure
      */
     public static ResultSet polar2CartesianSet(Connection conn, String query)
             throws SQLException {
