@@ -1,6 +1,6 @@
 /*
- * Copyright 2004-2014 H2 Group. Multiple-Licensed under the MPL 2.0,
- * and the EPL 1.0 (http://h2database.com/html/license.html).
+ * Copyright 2004-2022 H2 Group. Multiple-Licensed under the MPL 2.0,
+ * and the EPL 1.0 (https://h2database.com/html/license.html).
  * Initial Developer: H2 Group
  */
 package org.h2.test.db;
@@ -13,6 +13,7 @@ import java.io.InputStream;
 import java.io.OutputStream;
 import java.io.Reader;
 import java.io.StringReader;
+import java.nio.charset.StandardCharsets;
 import java.sql.Blob;
 import java.sql.Clob;
 import java.sql.Connection;
@@ -22,25 +23,34 @@ import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Savepoint;
 import java.sql.Statement;
+import java.sql.Types;
+import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Random;
 import java.util.concurrent.TimeUnit;
 
 import org.h2.api.ErrorCode;
+import org.h2.engine.Constants;
 import org.h2.engine.SysProperties;
 import org.h2.jdbc.JdbcConnection;
 import org.h2.message.DbException;
+import org.h2.store.FileLister;
 import org.h2.store.fs.FileUtils;
 import org.h2.test.TestBase;
+import org.h2.test.TestDb;
 import org.h2.tools.Recover;
+import org.h2.tools.SimpleResultSet;
 import org.h2.util.IOUtils;
 import org.h2.util.JdbcUtils;
-import org.h2.util.StringUtils;
 import org.h2.util.Task;
+import org.h2.value.ValueBlob;
+import org.h2.value.ValueClob;
+import org.h2.value.ValueLob;
 
 /**
  * Tests LOB and CLOB data types.
  */
-public class TestLob extends TestBase {
+public class TestLob extends TestDb {
 
     private static final String MORE_THAN_128_CHARS =
             "12345678901234567890123456789012345678901234567890" +
@@ -55,16 +65,16 @@ public class TestLob extends TestBase {
     public static void main(String... a) throws Exception {
         TestBase test = TestBase.createCaller().init();
         test.config.big = true;
-        test.test();
+        test.testFromMain();
     }
 
     @Override
     public void test() throws Exception {
+        testReclamationOnInDoubtRollback();
         testRemoveAfterDeleteAndClose();
         testRemovedAfterTimeout();
         testConcurrentRemoveRead();
         testCloseLobTwice();
-        testCleaningUpLobsOnRollback();
         testClobWithRandomUnicodeChars();
         testCommitOnExclusiveConnection();
         testReadManyLobs();
@@ -74,7 +84,6 @@ public class TestLob extends TestBase {
         testBlobInputStreamSeek(true);
         testBlobInputStreamSeek(false);
         testDeadlock();
-        testDeadlock2();
         testCopyManyLobs();
         testCopyLob();
         testConcurrentCreate();
@@ -82,7 +91,6 @@ public class TestLob extends TestBase {
         testUniqueIndex();
         testConvert();
         testCreateAsSelect();
-        testDelete();
         testLobServerMemory();
         testUpdatingLobRow();
         testBufferedInputStreamBug();
@@ -90,7 +98,6 @@ public class TestLob extends TestBase {
             return;
         }
         testLargeClob();
-        testLobCleanupSessionTemporaries();
         testLobUpdateMany();
         testLobVariable();
         testLobDrop();
@@ -100,10 +107,7 @@ public class TestLob extends TestBase {
         testLobRollbackStop();
         testLobCopy();
         testLobHibernate();
-        testLobCopy(false);
-        testLobCopy(true);
-        testLobCompression(false);
-        testLobCompression(true);
+        testLobCopy2();
         testManyLobs();
         testClob();
         testUpdateLob();
@@ -111,9 +115,65 @@ public class TestLob extends TestBase {
         testLob(false);
         testLob(true);
         testJavaObject();
+        testLobInValueResultSet();
+        testLimits();
         deleteDb("lob");
     }
 
+    private void testReclamationOnInDoubtRollback() throws Exception {
+        if (config.memory || config.cipher != null) {
+            return;
+        }
+        deleteDb("lob");
+        try (Connection conn = getConnection("lob")) {
+            try (Statement st = conn.createStatement()) {
+                st.executeUpdate("CREATE TABLE IF NOT EXISTS dataTable("
+                        + "dataStamp BIGINT PRIMARY KEY, "
+                        + "data BLOB)");
+            }
+
+            conn.setAutoCommit(false);
+            Random rnd = new Random(0);
+            try (PreparedStatement pstmt = conn.prepareStatement("INSERT INTO dataTable VALUES(?, ?)")) {
+                for (int i = 0; i < 100; ++i) {
+                    int numBytes = 1024 * 1024;
+                    byte[] data = new byte[numBytes];
+                    rnd.nextBytes(data);
+                    pstmt.setLong(1, i);
+                    pstmt.setBytes(2, data);
+                    pstmt.executeUpdate();
+                }
+            }
+            try (Statement st = conn.createStatement()) {
+                st.executeUpdate("PREPARE COMMIT lobtx");
+                st.execute("SHUTDOWN IMMEDIATELY");
+            }
+        }
+
+        try (Connection conn = getConnection("lob")) {
+            try (Statement st = conn.createStatement();
+                    ResultSet rs = st.executeQuery("SELECT * FROM INFORMATION_SCHEMA.IN_DOUBT")) {
+                assertTrue("No in-doubt tx", rs.first());
+                assertEquals("LOBTX", rs.getString("TRANSACTION_NAME"));
+                assertFalse("more than one in-doubt tx", rs.next());
+                st.executeUpdate("ROLLBACK TRANSACTION lobtx; CHECKPOINT SYNC");
+            }
+        }
+
+        try (Connection conn = getConnection("lob")) {
+            try (Statement st = conn.createStatement()) {
+                st.execute("SHUTDOWN COMPACT");
+            }
+        }
+
+        ArrayList<String> dbFiles = FileLister.getDatabaseFiles(getBaseDir(), "lob", false);
+        assertEquals(1, dbFiles.size());
+        File file = new File(dbFiles.get(0));
+        assertTrue(file.exists());
+        long fileSize = file.length();
+        assertTrue("File size=" + fileSize, fileSize < 13000);
+    }
+
     private void testRemoveAfterDeleteAndClose() throws Exception {
         if (config.memory || config.cipher != null) {
             return;
@@ -167,7 +227,7 @@ public class TestLob extends TestBase {
             return;
         }
         deleteDb("lob");
-        final String url = getURL("lob;lob_timeout=50", true);
+        final String url = getURL("lob;lob_timeout=200", true);
         Connection conn = getConnection(url);
         Statement stat = conn.createStatement();
         stat.execute("create table test(id int primary key, data clob)");
@@ -189,15 +249,10 @@ public class TestLob extends TestBase {
         stat.execute("delete from test");
         c1.getSubString(1, 3);
         // wait until it times out
-        Thread.sleep(100);
+        Thread.sleep(250);
         // start a new transaction, to be sure
         stat.execute("delete from test");
-        try {
-            c1.getSubString(1, 3);
-            fail();
-        } catch (SQLException e) {
-            // expected
-        }
+        assertThrows(SQLException.class, c1).getSubString(1, 3);
         conn.close();
     }
 
@@ -237,28 +292,6 @@ public class TestLob extends TestBase {
         conn.close();
     }
 
-    private void testCleaningUpLobsOnRollback() throws Exception {
-        if (config.mvStore) {
-            return;
-        }
-        deleteDb("lob");
-        Connection conn = getConnection("lob");
-        Statement stat = conn.createStatement();
-        stat.execute("CREATE TABLE test(id int, data CLOB)");
-        conn.setAutoCommit(false);
-        stat.executeUpdate("insert into test values (1, '" +
-                MORE_THAN_128_CHARS + "')");
-        conn.rollback();
-        ResultSet rs = stat.executeQuery("select count(*) from test");
-        rs.next();
-        assertEquals(0, rs.getInt(1));
-        rs = stat.executeQuery("select * from information_schema.lobs");
-        rs = stat.executeQuery("select count(*) from information_schema.lob_data");
-        rs.next();
-        assertEquals(0, rs.getInt(1));
-        conn.close();
-    }
-
     private void testReadManyLobs() throws Exception {
         deleteDb("lob");
         Connection conn;
@@ -266,7 +299,7 @@ public class TestLob extends TestBase {
         Statement stat = conn.createStatement();
         stat.execute("create table test(id identity, data clob)");
         PreparedStatement prep = conn.prepareStatement(
-                "insert into test values(null, ?)");
+                "insert into test(data) values ?");
         byte[] data = new byte[256];
         Random r = new Random(1);
         for (int i = 0; i < 1000; i++) {
@@ -365,17 +398,6 @@ public class TestLob extends TestBase {
             prep.setBinaryStream(2, new ByteArrayInputStream(buff), -1);
             prep.execute();
         }
-        if (upgraded) {
-            if (!config.mvStore) {
-                if (config.memory) {
-                    stat.execute("update information_schema.lob_map set pos=null");
-                } else {
-                    stat.execute("alter table information_schema.lob_map drop column pos");
-                    conn.close();
-                    conn = getConnection("lob");
-                }
-            }
-        }
         prep = conn.prepareStatement("select * from test where id = ?");
         for (int i = 0; i < 1; i++) {
             random.setSeed(i);
@@ -430,125 +452,20 @@ public class TestLob extends TestBase {
         conn2.close();
     }
 
-    /**
-     * A background task.
-     */
-    private final class Deadlock2Task1 extends Task {
-
-        public final Connection conn;
-
-        Deadlock2Task1() throws SQLException {
-            this.conn = getDeadlock2Connection();
-        }
-
-        @Override
-        public void call() throws Exception {
-            Random random = new Random();
-            Statement stat = conn.createStatement();
-            char[] tmp = new char[1024];
-            while (!stop) {
-                try {
-                    ResultSet rs = stat.executeQuery(
-                            "select name from test where id = " + random.nextInt(999));
-                    if (rs.next()) {
-                        Reader r = rs.getClob("name").getCharacterStream();
-                        while (r.read(tmp) >= 0) {
-                            // ignore
-                        }
-                        r.close();
-                    }
-                    rs.close();
-                } catch (SQLException ex) {
-                    // ignore "LOB gone away", this can happen
-                    // in the presence of concurrent updates
-                    if (ex.getErrorCode() != ErrorCode.IO_EXCEPTION_2) {
-                        throw ex;
-                    }
-                } catch (IOException ex) {
-                    // ignore "LOB gone away", this can happen
-                    // in the presence of concurrent updates
-                    Exception e = ex;
-                    if (e.getCause() instanceof DbException) {
-                        e = (Exception) e.getCause();
-                    }
-                    if (!(e.getCause() instanceof SQLException)) {
-                        throw ex;
-                    }
-                    SQLException e2 = (SQLException) e.getCause();
-                    if (e2.getErrorCode() != ErrorCode.IO_EXCEPTION_1) {
-                        throw ex;
-                    }
-                } catch (Exception e) {
-                    e.printStackTrace(System.out);
-                    throw e;
-                }
-            }
-        }
-
-    }
-
-    /**
-     * A background task.
-     */
-    private final class Deadlock2Task2 extends Task {
-
-        public final Connection conn;
-
-        Deadlock2Task2() throws SQLException {
-            this.conn = getDeadlock2Connection();
-        }
-
-        @Override
-        public void call() throws Exception {
-            Random random = new Random();
-            Statement stat = conn.createStatement();
-            while (!stop) {
-                stat.execute("update test set counter = " +
-                        random.nextInt(10) + " where id = " + random.nextInt(1000));
-            }
-        }
-
-    }
-
-    private void testDeadlock2() throws Exception {
-        if (config.mvcc || config.memory) {
-            return;
-        }
-        deleteDb("lob");
-        Connection conn = getDeadlock2Connection();
-        Statement stat = conn.createStatement();
-        stat.execute("create cached table test(id int not null identity, " +
-                "name clob, counter int)");
-        stat.execute("insert into test(id, name) select x, space(100000) " +
-                "from system_range(1, 100)");
-        Deadlock2Task1 task1 = new Deadlock2Task1();
-        Deadlock2Task2 task2 = new Deadlock2Task2();
-        task1.execute("task1");
-        task2.execute("task2");
-        for (int i = 0; i < 100; i++) {
-            stat.execute("insert into test values(null, space(10000 + " + i + "), 1)");
-        }
-        task1.get();
-        task1.conn.close();
-        task2.get();
-        task2.conn.close();
-        conn.close();
-    }
-
     Connection getDeadlock2Connection() throws SQLException {
-        return getConnection("lob;MULTI_THREADED=TRUE;LOCK_TIMEOUT=60000");
+        return getConnection("lob;LOCK_TIMEOUT=60000");
     }
 
     private void testCopyManyLobs() throws Exception {
         deleteDb("lob");
         Connection conn = getConnection("lob");
         Statement stat = conn.createStatement();
-        stat.execute("create table test(id identity, data clob) " +
-                "as select 1, space(10000)");
-        stat.execute("insert into test(id, data) select null, data from test");
-        stat.execute("insert into test(id, data) select null, data from test");
-        stat.execute("insert into test(id, data) select null, data from test");
-        stat.execute("insert into test(id, data) select null, data from test");
+        stat.execute("create table test(id identity default on null, data clob) " +
+                "as select null, space(10000)");
+        stat.execute("insert into test(data) select data from test");
+        stat.execute("insert into test(data) select data from test");
+        stat.execute("insert into test(data) select data from test");
+        stat.execute("insert into test(data) select data from test");
         stat.execute("delete from test where id < 10");
         stat.execute("shutdown compact");
         conn.close();
@@ -641,12 +558,7 @@ public class TestLob extends TestBase {
         Statement stat;
         conn = getConnection("lob");
         stat = conn.createStatement();
-        try {
-            stat.execute("create memory table test(x clob unique)");
-            fail();
-        } catch (SQLException e) {
-            assertEquals(ErrorCode.FEATURE_NOT_SUPPORTED_1, e.getErrorCode());
-        }
+        assertThrows(ErrorCode.FEATURE_NOT_SUPPORTED_1, stat).execute("create memory table test(x clob unique)");
         conn.close();
     }
 
@@ -683,47 +595,6 @@ public class TestLob extends TestBase {
         conn.close();
     }
 
-    private void testDelete() throws Exception {
-        if (config.memory || config.mvStore) {
-            return;
-        }
-        deleteDb("lob");
-        Connection conn;
-        Statement stat;
-        conn = getConnection("lob");
-        stat = conn.createStatement();
-        stat.execute("create table test(id int primary key, name clob)");
-        stat.execute("insert into test values(1, space(10000))");
-        assertSingleValue(stat,
-                "select count(*) from information_schema.lob_data", 1);
-        stat.execute("insert into test values(2, space(10000))");
-        assertSingleValue(stat,
-                "select count(*) from information_schema.lob_data", 1);
-        stat.execute("delete from test where id = 1");
-        assertSingleValue(stat,
-                "select count(*) from information_schema.lob_data", 1);
-        stat.execute("insert into test values(3, space(10000))");
-        assertSingleValue(stat,
-                "select count(*) from information_schema.lob_data", 1);
-        stat.execute("insert into test values(4, space(10000))");
-        assertSingleValue(stat,
-                "select count(*) from information_schema.lob_data", 1);
-        stat.execute("delete from test where id = 2");
-        assertSingleValue(stat,
-                "select count(*) from information_schema.lob_data", 1);
-        stat.execute("delete from test where id = 3");
-        assertSingleValue(stat,
-                "select count(*) from information_schema.lob_data", 1);
-        stat.execute("delete from test");
-        conn.close();
-        conn = getConnection("lob");
-        stat = conn.createStatement();
-        assertSingleValue(stat,
-                "select count(*) from information_schema.lob_data", 0);
-        stat.execute("drop table test");
-        conn.close();
-    }
-
     private void testLobUpdateMany() throws SQLException {
         deleteDb("lob");
         Connection conn = getConnection("lob");
@@ -736,35 +607,6 @@ public class TestLob extends TestBase {
         conn.close();
     }
 
-    private void testLobCleanupSessionTemporaries() throws SQLException {
-        if (config.mvStore) {
-            return;
-        }
-        deleteDb("lob");
-        Connection conn = getConnection("lob");
-        Statement stat = conn.createStatement();
-        stat.execute("create table test(data clob)");
-
-        ResultSet rs = stat.executeQuery("select count(*) " +
-                "from INFORMATION_SCHEMA.LOBS");
-        assertTrue(rs.next());
-        assertEquals(0, rs.getInt(1));
-        rs.close();
-
-        PreparedStatement prep = conn.prepareStatement(
-                "INSERT INTO test(data) VALUES(?)");
-        String name = new String(new char[200]).replace((char) 0, 'x');
-        prep.setString(1, name);
-        prep.execute();
-        prep.close();
-
-        rs = stat.executeQuery("select count(*) from INFORMATION_SCHEMA.LOBS");
-        assertTrue(rs.next());
-        assertEquals(1, rs.getInt(1));
-        rs.close();
-        conn.close();
-    }
-
     private void testLobServerMemory() throws SQLException {
         deleteDb("lob");
         Connection conn = getConnection("lob");
@@ -1073,39 +915,46 @@ public class TestLob extends TestBase {
         prep2.getQueryTimeout();
         prep2.close();
         conn0.getAutoCommit();
-        Reader r = clob0.getCharacterStream();
+        Reader r;
+        int ch;
+        r = clob0.getCharacterStream();
         for (int i = 0; i < 10000; i++) {
-            int ch = r.read();
+            ch = r.read();
             if (ch != ('0' + (i % 10))) {
                 fail("expected " + (char) ('0' + (i % 10)) +
                         " got: " + ch + " (" + (char) ch + ")");
             }
         }
-        int ch = r.read();
+        ch = r.read();
         if (ch != -1) {
             fail("expected -1 got: " + ch);
         }
+        r.close();
+        r = clob0.getCharacterStream(1235, 1000);
+        for (int i = 1234; i < 2234; i++) {
+            ch = r.read();
+            if (ch != ('0' + (i % 10))) {
+                fail("expected " + (char) ('0' + (i % 10)) +
+                        " got: " + ch + " (" + (char) ch + ")");
+            }
+        }
+        ch = r.read();
+        if (ch != -1) {
+            fail("expected -1 got: " + ch);
+        }
+        r.close();
+        assertThrows(ErrorCode.INVALID_VALUE_2, clob0).getCharacterStream(10001, 1);
+        assertThrows(ErrorCode.INVALID_VALUE_2, clob0).getCharacterStream(10002, 0);
         conn0.close();
     }
 
-    private void testLobCopy(boolean compress) throws SQLException {
+    private void testLobCopy2() throws SQLException {
         deleteDb("lob");
         Connection conn;
         conn = reconnect(null);
         Statement stat = conn.createStatement();
-        if (compress) {
-            stat.execute("SET COMPRESS_LOB LZF");
-        } else {
-            stat.execute("SET COMPRESS_LOB NO");
-        }
         conn = reconnect(conn);
         stat = conn.createStatement();
-        ResultSet rs;
-        rs = stat.executeQuery("select value from information_schema.settings " +
-                "where NAME='COMPRESS_LOB'");
-        rs.next();
-        assertEquals(compress ? "LZF" : "NO", rs.getString(1));
-        assertFalse(rs.next());
         stat.execute("create table test(text clob)");
         stat.execute("create table test2(text clob)");
         StringBuilder buff = new StringBuilder();
@@ -1115,7 +964,7 @@ public class TestLob extends TestBase {
         String spaces = buff.toString();
         stat.execute("insert into test values('" + spaces + "')");
         stat.execute("insert into test2 select * from test");
-        rs = stat.executeQuery("select * from test2");
+        ResultSet rs = stat.executeQuery("select * from test2");
         rs.next();
         assertEquals(spaces, rs.getString(1));
         stat.execute("drop table test");
@@ -1129,55 +978,6 @@ public class TestLob extends TestBase {
         conn.close();
     }
 
-    private void testLobCompression(boolean compress) throws Exception {
-        deleteDb("lob");
-        Connection conn;
-        conn = reconnect(null);
-        if (compress) {
-            conn.createStatement().execute("SET COMPRESS_LOB LZF");
-        } else {
-            conn.createStatement().execute("SET COMPRESS_LOB NO");
-        }
-        conn.createStatement().execute("CREATE TABLE TEST(ID INT PRIMARY KEY, C CLOB)");
-        PreparedStatement prep = conn.prepareStatement(
-                "INSERT INTO TEST VALUES(?, ?)");
-        long time = System.nanoTime();
-        int len = getSize(10, 40);
-        if (config.networked && config.big) {
-            len = 5;
-        }
-        StringBuilder buff = new StringBuilder();
-        for (int i = 0; i < 1000; i++) {
-            buff.append(StringUtils.xmlNode("content", null, "This is a test " + i));
-        }
-        String xml = buff.toString();
-        for (int i = 0; i < len; i++) {
-            prep.setInt(1, i);
-            prep.setString(2, xml + i);
-            prep.execute();
-        }
-        for (int i = 0; i < len; i++) {
-            ResultSet rs = conn.createStatement().executeQuery(
-                    "SELECT * FROM TEST");
-            while (rs.next()) {
-                if (i == 0) {
-                    assertEquals(xml + rs.getInt(1), rs.getString(2));
-                } else {
-                    Reader r = rs.getCharacterStream(2);
-                    String result = IOUtils.readStringAndClose(r, -1);
-                    assertEquals(xml + rs.getInt(1), result);
-                }
-            }
-        }
-        time = System.nanoTime() - time;
-        trace("time: " + TimeUnit.NANOSECONDS.toMillis(time) + " compress: " + compress);
-        conn.close();
-        if (!config.memory) {
-            long length = new File(getBaseDir() + "/lob.h2.db").length();
-            trace("len: " + length + " compress: " + compress);
-        }
-    }
-
     private void testManyLobs() throws Exception {
         deleteDb("lob");
         Connection conn;
@@ -1358,7 +1158,7 @@ public class TestLob extends TestBase {
         PreparedStatement prep;
         prep = conn.prepareStatement("INSERT INTO TEST VALUES(1, ?)");
         String s = new String(getRandomChars(10000, 1));
-        byte[] data = s.getBytes("UTF-8");
+        byte[] data = s.getBytes(StandardCharsets.UTF_8);
         // if we keep the string, debugging with Eclipse is not possible
         // because Eclipse wants to display the large string and fails
         s = "";
@@ -1396,7 +1196,7 @@ public class TestLob extends TestBase {
         PreparedStatement prep;
         ResultSet rs;
         long time;
-        stat.execute("CREATE TABLE TEST(ID INT PRIMARY KEY, VALUE " +
+        stat.execute("CREATE TABLE TEST(ID INT PRIMARY KEY, V " +
                 (clob ? "CLOB" : "BLOB") + ")");
 
         int len = getSize(1, 1000);
@@ -1421,7 +1221,7 @@ public class TestLob extends TestBase {
         conn = reconnect(conn);
 
         time = System.nanoTime();
-        prep = conn.prepareStatement("SELECT ID, VALUE FROM TEST");
+        prep = conn.prepareStatement("SELECT ID, V FROM TEST");
         rs = prep.executeQuery();
         while (rs.next()) {
             int id = rs.getInt("ID");
@@ -1502,13 +1302,13 @@ public class TestLob extends TestBase {
         assertFalse(rs.next());
 
         conn.createStatement().execute("drop table test");
-        stat.execute("create table test(value other)");
+        stat.execute("create table test(v other)");
         prep = conn.prepareStatement("insert into test values(?)");
-        prep.setObject(1, JdbcUtils.serialize("", conn.getSession().getDataHandler()));
+        prep.setObject(1, JdbcUtils.serialize("", conn.getJavaObjectSerializer()));
         prep.execute();
-        rs = stat.executeQuery("select value from test");
+        rs = stat.executeQuery("select v from test");
         while (rs.next()) {
-            assertEquals("", (String) rs.getObject("value"));
+            assertEquals("", (String) rs.getObject("v"));
         }
         conn.close();
     }
@@ -1610,7 +1410,7 @@ public class TestLob extends TestBase {
         stat.execute("CREATE TABLE logs" +
                 "(id int primary key auto_increment, message CLOB)");
         PreparedStatement s1 = conn.prepareStatement(
-                "INSERT INTO logs (id, message) VALUES(null, ?)");
+                "INSERT INTO logs (message) VALUES ?");
         final Random rand = new Random(1);
         for (int i = 1; i <= 100; i++) {
             String data = randomUnicodeString(rand);
@@ -1664,4 +1464,118 @@ public class TestLob extends TestBase {
         }
         return new String(buffer);
     }
+
+    private void testLobInValueResultSet() throws SQLException {
+        deleteDb("lob");
+        JdbcConnection conn = (JdbcConnection) getConnection("lob");
+        Statement stat = conn.createStatement();
+        stat.execute("CREATE ALIAS VRS FOR '" + getClass().getName() + ".testLobInValueResultSetGet'");
+        ResultSet rs = stat.executeQuery("SELECT * FROM VRS()");
+        assertTrue(rs.next());
+        Clob clob = rs.getClob(1);
+        assertFalse(rs.next());
+        assertEquals(MORE_THAN_128_CHARS, clob.getSubString(1, Integer.MAX_VALUE));
+        conn.close();
+    }
+
+    /**
+     * This method is called via reflection from the database.
+     *
+     * @param conn connection
+     * @return the result set
+     * @throws SQLException on exception
+     */
+    public static SimpleResultSet testLobInValueResultSetGet(Connection conn) throws SQLException {
+        final Clob c = conn.createClob();
+        c.setString(1, MORE_THAN_128_CHARS);
+        SimpleResultSet rs = new SimpleResultSet() {
+            @Override
+            public Object getObject(int columnIndex) throws SQLException {
+                return c;
+            }
+        };
+        rs.addColumn("L", Types.CLOB, 1000, 0);
+        rs.addRow(MORE_THAN_128_CHARS);
+        return rs;
+    }
+
+    private void testLimits() throws Exception {
+        deleteDb("lob");
+        JdbcConnection conn = (JdbcConnection) getConnection("lob");
+        Statement stat = conn.createStatement();
+        stat.execute("CREATE TABLE TEST(ID INTEGER, B BLOB, C CLOB)");
+        PreparedStatement ps = conn.prepareStatement("INSERT INTO TEST VALUES (?, ?, ?)");
+        ps.setInt(1, 1);
+        byte[] b = new byte[Constants.MAX_STRING_LENGTH];
+        Arrays.fill(b, (byte) 'A');
+        String s = new String(b, StandardCharsets.UTF_8);
+        ps.setBytes(2, b);
+        ps.setString(3, s);
+        ps.executeUpdate();
+        byte[] b2 = new byte[Constants.MAX_STRING_LENGTH + 1];
+        Arrays.fill(b2, (byte) 'A');
+        String s2 = new String(b2, StandardCharsets.UTF_8);
+        assertThrows(ErrorCode.VALUE_TOO_LONG_2, ps).setBytes(2, b2);
+        ps.setBinaryStream(2, new ByteArrayInputStream(b2));
+        assertThrows(ErrorCode.VALUE_TOO_LONG_2, ps).setString(3, s2);
+        ps.setCharacterStream(3, new StringReader(s2));
+        ps.executeUpdate();
+        try (ResultSet rs = stat.executeQuery("TABLE TEST ORDER BY ID")) {
+            assertTrue(rs.next());
+            assertEquals(1, rs.getInt(1));
+            testLimitsSmall(b, s, rs, 2);
+            testLimitsSmall(b, s, rs, 2);
+            testLimitsSmall(b, s, rs, 3);
+            testLimitsSmall(b, s, rs, 3);
+            assertTrue(rs.next());
+            assertEquals(1, rs.getInt(1));
+            testLimitsLarge(b2, s2, rs, 2);
+            testLimitsLarge(b2, s2, rs, 2);
+            testLimitsLarge(b2, s2, rs, 3);
+            testLimitsLarge(b2, s2, rs, 3);
+            assertFalse(rs.next());
+        }
+        conn.close();
+        testLimitsSmall(b, s, ValueBlob.createSmall(b));
+        testLimitsSmall(b, s, ValueClob.createSmall(b, Constants.MAX_STRING_LENGTH));
+        testLimitsLarge(b2, s2, ValueBlob.createSmall(b2));
+        testLimitsLarge(b2, s2, ValueClob.createSmall(b2, Constants.MAX_STRING_LENGTH + 1));
+    }
+
+    private void testLimitsSmall(byte[] b, String s, ResultSet rs, int index) throws SQLException {
+        assertEquals(b, rs.getBytes(index));
+        assertEquals(s, rs.getString(index));
+    }
+
+    private void testLimitsLarge(byte[] b, String s, ResultSet rs, int index) throws SQLException, IOException {
+        assertThrows(ErrorCode.VALUE_TOO_LONG_2, rs).getBytes(index);
+        assertEquals(b, IOUtils.readBytesAndClose(rs.getBlob(index).getBinaryStream(), -1));
+        assertThrows(ErrorCode.VALUE_TOO_LONG_2, rs).getString(index);
+        assertEquals(s, IOUtils.readStringAndClose(rs.getClob(index).getCharacterStream(), -1));
+    }
+
+    private void testLimitsSmall(byte[] b, String s, ValueLob v) {
+        assertEquals(b, v.getBytesNoCopy());
+        assertEquals(s, v.getString());
+        assertEquals(s, v.getString());
+    }
+
+    private void testLimitsLarge(byte[] b, String s, ValueLob v) throws IOException {
+        try {
+            assertEquals(b, v.getBytesNoCopy());
+            throw new AssertionError();
+        } catch (DbException e) {
+            assertEquals(ErrorCode.VALUE_TOO_LONG_2, e.getErrorCode());
+        }
+        assertEquals(b, IOUtils.readBytesAndClose(v.getInputStream(), -1));
+        for (int i = 0; i < 2; i++) {
+            try {
+                assertEquals(s, v.getString());
+                throw new AssertionError();
+            } catch (DbException e) {
+                assertEquals(ErrorCode.VALUE_TOO_LONG_2, e.getErrorCode());
+            }
+            assertEquals(s, IOUtils.readStringAndClose(v.getReader(), -1));
+        }
+    }
 }
